##############################################################################################
# THE FOLLOWING EXAMPLE CODE IS INTENDED FOR LIMITED CIRCULATION ONLY.
# 
# Please forward all questions regarding this code to ANT Technical Support.
# 
# Dynastream Innovations Inc.
# 228 River Avenue
# Cochrane, Alberta, Canada
# T4C 2C1
# 
# (P) (403) 932-9292
# (F) (403) 932-6521
# (TF) 1-866-932-9292
# (E) support@thisisant.com
# 
# www.thisisant.com
#
# Reference Design Disclaimer
#
# The references designs and codes provided may be used with ANT devices only and remain the copyrighted property of 
# Dynastream Innovations Inc. The reference designs and codes are being provided on an "as-is" basis and as an accommodation, 
# and therefore all warranties, representations, or guarantees of any kind (whether express, implied or statutory) including, 
# without limitation, warranties of merchantability, non-infringement,
# or fitness for a particular purpose, are specifically disclaimed.
#
# 2010 Dynastream Innovations Inc. All Rights Reserved
# This software may not be reproduced by
# any means without express written approval of Dynastream
# Innovations Inc.
#
##############################################################################################
use strict;

##############################################
# Message ID's
##############################################
use constant MESG_INVALID_ID                   => '00';
use constant MESG_EVENT_ID                     => '01';

use constant MESG_VERSION_ID                   => '3E';  
use constant MESG_RESPONSE_EVENT_ID            => '40';

use constant MESG_UNASSIGN_CHANNEL_ID          => '41';
use constant MESG_ASSIGN_CHANNEL_ID            => '42';
use constant MESG_CHANNEL_MESG_PERIOD_ID       => '43';
use constant MESG_CHANNEL_SEARCH_TIMEOUT_ID    => '44';
use constant MESG_CHANNEL_RADIO_FREQ_ID        => '45';
use constant MESG_NETWORK_KEY_ID               => '46';
use constant MESG_RADIO_TX_POWER_ID            => '47';
use constant MESG_RADIO_CW_MODE_ID             => '48';
use constant MESG_SEARCH_WAVEFORM_ID           => '49';

use constant MESG_SYSTEM_RESET_ID              => '4A';
use constant MESG_OPEN_CHANNEL_ID              => '4B';
use constant MESG_CLOSE_CHANNEL_ID             => '4C';
use constant MESG_REQUEST_ID                   => '4D';

use constant MESG_BROADCAST_DATA_ID            => '4E';
use constant MESG_ACKNOWLEDGED_DATA_ID         => '4F';
use constant MESG_BURST_DATA_ID                => '50';

use constant MESG_CHANNEL_ID_ID                => '51';
use constant MESG_CHANNEL_STATUS_ID            => '52';
use constant MESG_RADIO_CW_INIT_ID             => '53';
use constant MESG_CAPABILITIES_ID              => '54';

use constant MESG_NVM_DATA_ID                  => '56';
use constant MESG_NVM_CMD_ID                   => '57';
use constant MESG_NVM_STRING_ID                => '58';

use constant MESG_ID_LIST_ADD_ID               => '59';
use constant MESG_ID_LIST_CONFIG_ID            => '5A';
use constant MESG_OPEN_RX_SCAN_ID              => '5B';

use constant MESG_EXT_CHANNEL_RADIO_FREQ_ID    => '5C';
use constant MESG_EXT_BROADCAST_DATA_ID        => '5D';
use constant MESG_EXT_ACKNOWLEDGED_DATA_ID     => '5E';
use constant MESG_EXT_BURST_DATA_ID            => '5F';

use constant MESG_CHANNEL_RADIO_TX_POWER_ID    => '60';
use constant MESG_GET_SERIAL_NUM_ID            => '61';
use constant MESG_GET_TEMP_CAL_ID              => '62';
use constant MESG_SET_LP_SEARCH_TIMEOUT_ID     => '63';
use constant MESG_SET_TX_SEARCH_ON_NEXT_ID     => '64';
use constant MESG_SERIAL_NUM_SET_CHANNEL_ID_ID => '65';
use constant MESG_RX_EXT_MESGS_ENABLE_ID       => '66';
use constant MESG_RADIO_CONFIG_ALWAYS_ID       => '67';
use constant MESG_ENABLE_LED_FLASH_ID          => '68';

use constant MESG_AGC_CONFIG_ID                => '6A';

use constant MESG_PIN_DIODE_CONTROL_ID		   => '90';
use constant MESG_RUN_SCRIPT_ID                => '91';

use constant MESG_READ_SEGA_ID                 => 'A0';
use constant MESG_SEGA_CMD_ID                  => 'A1';
use constant MESG_SEGA_DATA_ID                 => 'A2';
use constant MESG_SEGA_ERASE_ID                => 'A3';
use constant MESG_SEGA_WRITE_ID                => 'A4';
use constant MESG_SEGA_LOCK_ID                 => 'A6';
use constant MESG_FUSECHECK_ID                 => 'A7';
use constant MESG_UARTREG_ID                   => 'A8';
use constant MESG_MAN_TEMP_ID                  => 'A9';
use constant MESG_BIST_ID                      => 'AA';
use constant MESG_SELFERASE_ID                 => 'AB';
use constant MESG_SET_MFG_BITS_ID              => 'AC';
use constant MESG_UNLOCK_INTERFACE_ID          => 'AD';

use constant MESG_IO_STATE_ID                  => 'B0';
use constant MESG_CFG_STATE_ID                 => 'B1';

use constant MESG_RSSI_ID                      => 'C0';
use constant MESG_RSSI_BROADCAST_DATA_ID       => 'C1';
use constant MESG_RSSI_ACKNOWLEDGED_DATA_ID    => 'C2';
use constant MESG_RSSI_BURST_DATA_ID           => 'C3';
use constant MESG_RSSI_SEARCH_THRESHOLD_ID     => 'C4';

use constant MESG_BTH_BROADCAST_DATA_ID        => 'D0';
use constant MESG_BTH_ACKNOWLEDGED_DATA_ID     => 'D1';
use constant MESG_BTH_BURST_DATA_ID            => 'D2';

use constant MESG_BTH_EXT_BROADCAST_DATA_ID    => 'D3';
use constant MESG_BTH_EXT_ACKNOWLEDGED_DATA_ID => 'D4';
use constant MESG_BTH_EXT_BURST_DATA_ID        => 'D5';
use constant MESG_RUN_SCRIPT                   => '91';


# ANT-FS message ID constants
use constant MESGFS_ANTFS_COMMAND               => 0x44;
use constant MESGFS_ANTFS_BEACON                => 0x43;
use constant MESGFS_AUTHENTICATE_ID             => 0x04;
use constant MESGFS_AUTHENTICATE_RESPONSE_ID    => 0x84;
use constant MESGFS_LINK_ID                     => 0x02;
use constant MESGFS_DOWNLOAD_ID                 => 0x06;

my %fit1_script_lut = (           
        0 => 'FIT1_OFF', 
        1 => 'FE_READY',
        2 => 'FE_IN_USE',
        3 => 'FE_PAUSED'
    );


my %command_lut = (           
        2 => 'Link',          
        3 => 'Disconnect',    
        4 => 'Authenticate',
        5 => 'Ping',
        9 => 'Download', 
        10 => 'Upload', 
        11 => 'Erase' 
    );

my %beacon_period_lut = (           
        0 => '0.5Hz',          
        1 => '1Hz',    
        2 => '2Hz',
        3 => '4Hz',
        4 => '8Hz' 
    );

my %beacon_layer_lut = (           
        0 => 'LINK',          
        1 => 'AUTH',    
        2 => 'TRANS',
        3 => 'BUSY'
    );
    
    
my %command_auth_type = (
        0 => 'Proceed',
        1 => 'Request Serial Number',
        2 => 'Pairing',
        3 => 'Passkey'
     );    
     
my %command_auth_response = (
        0 => 'N/A',
        1 => 'Accept',
        2 => 'Reject'
     ); 
     
my %command_response_lut = (
        0 => 'RESPONSE_NO_ERROR',
        1 => 'EVENT_RX_SEARCH_TIMEOUT',
        2 => 'EVENT_RX_FAIL',
        3 => 'EVENT_TX',
        4 => 'EVENT_TRANSFER_RX_FAILED',
        5 => 'EVENT_TRANSFER_TX_COMPLETED',
        6 => 'EVENT_TRANSFER_TX_FAILED',
        7 => 'EVENT_CHANNEL_CLOSED',
        8 => 'EVENT_RX_FAIL_GO_TO_SEARCH',
        9 => 'EVENT_CHANNEL_COLLISION',
        10 => 'EVENT_TRANSFER_TX_START',
        16 => 'EVENT_TRANSFER_TX_COMPLETED_RSSI',
        21 => 'CHANNEL_IN_WRONG_STATE',
        22 => 'CHANNEL_NOT_OPENED',
        24 => 'CHANNEL_ID_NOT_SET',
        25 => 'CLOSE_ALL_CHANNELS',
        31 => 'TRANSFER_IN_PROGRESS',
        32 => 'TRANSFER_SEQUENCE_NUMBER_ERROR',
        33 => 'TRANSFER_IN_ERROR',
        34 => 'TRANSFER_BUSY',
        40 => 'INVALID_MESSAGE',
        41 => 'INVALID_NETWORK_NUMBER',
        48 => 'INVALID_LIST_ID',
        49 => 'INVALID_SCAN_TX_CHANNEL',
        64 => 'NVM_FULL_ERROR',
        65 => 'NVM_WRITE_ERROR',
        96 => 'FIT_ACTIVE_SEARCH_TIMEOUT'
        
     );    
   
my %message_count = 0;



                                                                   
    

main();                       


##############################################################################################
# Functon: main
#
# Descrption:
#
# Runs the program, entry point as called above.. 
# Params:
# N/A.
#
# returns: N/A. 
##############################################################################################
sub main() {

   my $line;
   my @raw_data = GetFileData();                                  # Get the data from the file and read into an array
   
   #printf "###ANT_SCRIPT_VERSION: 0.01 \n"
                                                 
   foreach $line (@raw_data)                                      # Parse each line of the file.
   {
      my @hexbytes = ParseToHexBytes($line);                      # @hexbytes is an array that contains the ascii hex bytes (A4 09 etc ..)
      my $txdata = MakeTxPacket(@hexbytes);
      #my $script_id = ParseToHexBytes_Front($line);

    

    
      {
         printf "$txdata"   
      }      
   }
}


##############################################################################################
# Functon: GetFileData
#
# Descrption: Read data from file, die gracefully if this fails. 
#
# Params: N/A
#
# returns:  Array of file lines. That is, each array element is a line of the file, first 
#           line is index 0 and so on 
##############################################################################################
sub GetFileData {

   my $file = $ARGV[0];                                           # Assume first arg is the file we want to parse.

   if($file eq "")
   {
      print "Usage: \n\nperl parse.pl deviceX.txt \n\n... where deviceX.txt is an ANT log file.";
      exit(1);
   }

   open(INF, $file) || die("Could not open file!");               # open the log file and read in the data and close the file
   my @raw_data=<INF>; 
   close(INF);
   return @raw_data;

}

##############################################################################################
# Functon: ParseToHexBytes
#
# Descrption: Parse a line from the file into an array of ascii hex bytes that contain the serial 
#              message only, starting with the sync byte. Example "A4","09", etc ... 
#
# Params: A string that is a line of a file.
#
# returns:  Array of hexbytes
##############################################################################################
sub ParseToHexBytes {

   my $pline = shift;
   my @record = split('-', $pline);                               # $record[1] now contains log hex bytes only eg: [A4][09] etc ...
   #my @front = split(' ', $record[0]);
   
   my @hexbytes = $record[1];
   
   my $string = substr($record[1], 2, length($record[1])-4);      # remove first and last square bracket
   my @hexbytes = split(/\]\[/, $string);                         # @hexbytes is an array that contains the ascii hex bytes (A4 09 etc ..)
   
   return @hexbytes;

}

##############################################################################################
# Functon: ParseToHexBytes_Front
#
# Descrption: Parse a line from the file into an array of ascii hex bytes that contain the serial 
#              message only, starting with the sync byte. Example "A4","09", etc ... 
#
# Params: A string that is a line of a file.
#
# returns:  Array of hexbytes
##############################################################################################
sub ParseToHexBytes_Front {

   my $pline = shift;
   my @record = split('-', $pline);                               # $record[1] now contains log hex bytes only eg: [A4][09] etc ...
   my @front = split(' ', $record[0]);
   my $returnval = "w";

  if (@front[3] eq "Rx")
  {
    $returnval = "r";
  }
  if (@front[3] eq "Tx")
  {
   #$returnval = "w";
  }
   return $returnval;
  
}

##############################################################################################
# Functon: MakeTxPacket
#
# Descrption: Parse a line from the file into an array of ascii hex bytes that contain the serial 
#              message only, starting with the sync byte. Example "A4","09", etc ... 
#
# Params: Array of hexbytes
#
# returns:  Array of hexbytes
##############################################################################################
sub MakeTxPacket {

    my $length = $_[1] ;
    my $message = $_[2] ;
    my $channel = $_[3];
    my $txpacket;


    if ($message eq MESG_BROADCAST_DATA_ID)
    {
        $txpacket = "w ";
        for (my $index = 0; $index <= $length; $index++)
        {
            $txpacket = $txpacket."[".$_[$index+2]."]";
        }
        
        $txpacket = $txpacket."\n";
        
        if( $channel eq '00')
        {
            $txpacket = $txpacket."r! [40][00][01][03] \n";
        }    
    }


   return $txpacket;
  
}





